from crispy_forms.bootstrap import FormActions, StrictButton
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Reset, Submit
from django import forms
from django.utils.translation import gettext_lazy as _
from django_filters import FilterSet


class BaseFormMixin:
    layout = None
    fields_layout = None
    form_actions_layout = None
    submit_btn_text = _("Submit")
    helper_attrs = {}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.customize()

    def customize(self, default_helper_attrs=None):
        self.helper = FormHelper(self)
        self.helper_attrs = {
            "include_media": False,
            **(default_helper_attrs or {}),
            **self.helper_attrs,
        }
        for key, value in self.helper_attrs.items():
            setattr(self.helper, key, value)

        if self.layout:
            self.helper.layout = self.layout
        else:
            self.helper.layout = Layout(
                self.fields_layout or self.helper.layout,
                FormActions(
                    self.form_actions_layout
                    or StrictButton(
                        self.submit_btn_text,
                        type="submit",
                        css_class="btn btn-primary float-right",
                    ),
                ),
            )

        extra_kwargs = {}
        if hasattr(self.__class__, "Meta"):
            extra_kwargs = getattr(self.__class__.Meta, "extra_kwargs", {})

        for field_name, field in self.fields.items():
            if isinstance(field, forms.ModelChoiceField):
                if field.empty_label == "---------":
                    field.empty_label = "Choose One"
            if field.label is not None:
                field.label = field.label.capitalize()
            if field_name in extra_kwargs:
                for key, value in extra_kwargs[field_name].items():
                    setattr(field, key, value)


class HorizontalFormMixin:
    def customize(self, default_helper_attrs=None):
        super().customize(
            {
                "form_class": "form-horizontal",
                "label_class": "col-sm-3",
                "field_class": "col-sm-9",
                **(default_helper_attrs or {}),
            }
        )


class VerticalForm(BaseFormMixin, forms.Form):
    pass


class HorizontalForm(HorizontalFormMixin, BaseFormMixin, forms.Form):
    pass


class VerticalModelForm(BaseFormMixin, forms.ModelForm):
    pass


class HorizontalModelForm(HorizontalFormMixin, BaseFormMixin, forms.ModelForm):
    pass


class FilterForm(FilterSet):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        labels = getattr(self.__class__.Meta, "labels", {})
        for field_name, field in self.form.fields.items():
            if field_name in labels:
                label = labels[field_name]
                label = label.field.verbose_name if hasattr(label, "field") else label
                field.label = label.capitalize()
            if isinstance(field, forms.NullBooleanField):
                field.widget.choices[0] = ("unknown", "Any")

    def get_form_class(self):
        form_class = super().get_form_class()
        helper = FormHelper()
        helper.form_method = "GET"
        helper.add_input(Submit("submit", _("Search")))
        helper.add_input(
            Reset(
                "reset",
                _("Clear Filters"),
                css_class="btn btn-danger",
                onclick=f'location.href="{self.request.path}"',
            )
        )
        form_class.helper = helper
        return form_class



from django.shortcuts import redirect
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _


class NextMixin:
    success_url = None

    def get(self, request, *args, **kwargs):
        next_url = request.GET.get('next', None)
        if next_url is None:
            http_referer = request.META.get('HTTP_REFERER', None)
            if http_referer is not None:
                http_referer = '/'.join(['', http_referer.split('/', 3).pop()])
            return redirect(request.path + '?next=' + (http_referer or ''))
        return super().get(request, *args, **kwargs)

    def get_success_url(self):
        return self.request.GET.get('next') or self.success_url or '/'


class TitleMixin:

    @property
    def list_title(self):
        return self.model._meta.verbose_name_plural

    @property
    def create_title(self):
        return format_lazy(
            _("Add new {entity}"),
            entity=self.model._meta.verbose_name,
        )

    @property
    def update_title(self):
        return format_lazy(
            _("Edit {entity}"),
            entity=self.model._meta.verbose_name,
        )

    @property
    def delete_title(self):
        return format_lazy(
            _("Delete {entity}"),
            entity=self.model._meta.verbose_name,
        )



from django.contrib import messages
from django.core.exceptions import ValidationError
from django.http import HttpResponseRedirect
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _
from django.views import generic
from django_tables2 import SingleTableView

from .mixins import NextMixin, TitleMixin


class ListView(TitleMixin, SingleTableView):
    template_name = "core/list.html"
    paginate_by = 30


class CreateView(TitleMixin, NextMixin, generic.CreateView):
    template_name = "core/edit.html"

    def post(self, request, *args, **kwargs):
        try:
            response = super().post(request, *args, **kwargs)
            if self.get_form().is_valid():
                messages.success(
                    self.request,
                    format_lazy(
                        _("{entity} created successfully."),
                        entity=self.model._meta.verbose_name,
                    ),
                )
            return response
        except ValidationError as error:
            messages.error(self.request, error.message)
        return HttpResponseRedirect(self.get_success_url())


class UpdateView(TitleMixin, NextMixin, generic.UpdateView):
    template_name = "core/edit.html"

    def post(self, request, *args, **kwargs):
        try:
            response = super().post(request, *args, **kwargs)
            if self.get_form().is_valid():
                messages.success(
                    self.request,
                    format_lazy(
                        _("{entity} updated successfully."),
                        entity=self.model._meta.verbose_name,
                    ),
                )
            return response
        except ValidationError as error:
            messages.error(self.request, error.message)
        return HttpResponseRedirect(self.get_success_url())


class DeleteView(TitleMixin, generic.DeleteView):
    def delete(self, request, *args, **kwargs):
        try:
            response = super().delete(request, *args, **kwargs)
            messages.success(
                self.request,
                format_lazy(
                    _("{entity} deleted successfully."),
                    entity=self.model._meta.verbose_name,
                ),
            )
            return response
        except ValidationError as error:
            messages.error(self.request, error.message)
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return self.request.META["HTTP_REFERER"]



import setuptools


def get_long_description():
    with open("README.rst") as file:
        return file.read()

setuptools.setup(
    name="django-crispy-views",
    version="1.0.0",
    description="crispy views for django.",
    long_description=get_long_description(),
    url="https://github.com/monim67/django-crispy-views",
    author="Munim Munna",
    author_email="monim67@yahoo.com",
    license="MIT",
    keywords="crispy, views",
    packages=["crispy_views"],
    install_requires=["django>=2.0", "django-crispy-forms>=1.8.1", "django-filter>=2.2.0", "django-tables2>=2.2.1"],
    python_requires=">=3",
    classifiers=[
        "Intended Audience :: Developers",
        "Environment :: Web Environment",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Framework :: Django",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 3.0",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Version Control :: Git",
        "Topic :: Utilities",
    ],
)
